@@ -6,22 +6,16 @@ use std::mem;
 #[derive(Clone)]
 pub struct Maintenance1 {
     handle: vk::Device,
-    fns: vk::KhrMaintenance1Fn,
+    fp: vk::KhrMaintenance1Fn,
 }
 
 impl Maintenance1 {
     pub fn new(instance: &Instance, device: &Device) -> Self {
-        let fns = vk::KhrMaintenance1Fn::load(|name| unsafe {
-            mem::transmute(instance.get_device_proc_addr(device.handle(), name.as_ptr()))
+        let handle = device.handle();
+        let fp = vk::KhrMaintenance1Fn::load(|name| unsafe {
+            mem::transmute(instance.get_device_proc_addr(handle, name.as_ptr()))
         });
-        Self {
-            handle: device.handle(),
-            fns,
-        }
-    }
-
-    pub fn name() -> &'static CStr {
-        vk::KhrMaintenance1Fn::name()
+        Self { handle, fp }
     }
 
     #[doc = "<https://www.khronos.org/registry/vulkan/specs/1.2-extensions/man/html/vkTrimCommandPoolKHR.html>"]
@@ -30,12 +24,16 @@ impl Maintenance1 {
         command_pool: vk::CommandPool,
         flags: vk::CommandPoolTrimFlagsKHR,
     ) {
-        self.fns
+        self.fp
             .trim_command_pool_khr(self.handle, command_pool, flags);
     }
 
+    pub fn name() -> &'static CStr {
+        vk::KhrMaintenance1Fn::name()
+    }
+
     pub fn fp(&self) -> &vk::KhrMaintenance1Fn {
-        &self.fns
+        &self.fp
     }
 
     pub fn device(&self) -> vk::Device {
